// CS370 - Fall 2014
// Lab21 - DirtBall

#ifdef OSX
	#include <GLUT/glut.h>
#else
	#include <GL/glew.h>
	#include <GL/glut.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <SOIL/SOIL.h>

// Shader file utility functions
#include "shaderutils.h"

// Shader files
GLchar* vertexFile = "dirtvert.vs";
GLchar* fragmentFile = "dirtfrag.fs";

// Texture globals
#define NO_TEXTURES 3
#define BALL_UNIT 0
#define DIRT_UNIT 1
#define BALL 0
#define BLANK 1
#define DIRT 2
GLuint tex_ids[NO_TEXTURES];
char texture_files[NO_TEXTURES][20] = {"Basketball.bmp","blank.png","dirt.png"};

// Shader objects
GLuint dirtProg;
GLint texSampler[NO_TEXTURES];

// Scene globals
GLfloat theta[] = {30.0f,45.0f,0.0f};
int startx = 0;
int starty = 0;
bool add_dirt = false;

void display();
void render_Scene();
void keyfunc(unsigned char key, int x, int y);
void reshape(int w, int h);
void mousefunc(int button, int state, int x, int y);
void mousemove(int x, int y);
bool load_textures();

int main(int argc, char *argv[])
{
	// Initialize GLUT
	glutInit(&argc,argv);

	// Initialize the window with double buffering and RGB colors
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

	// Set the window size to image size
	glutInitWindowSize(512,512);

	// Create window
	glutCreateWindow("Dirty Basketball");

#ifndef OSX
	// Initialize GLEW - MUST BE DONE AFTER CREATING GLUT WINDOW
	glewInit();
#endif

	// Define callbacks
	glutDisplayFunc(display);
	glutKeyboardFunc(keyfunc);
	glutReshapeFunc(reshape);
	glutMouseFunc(mousefunc);
	glutMotionFunc(mousemove);

	// Set background color to white
	glClearColor(1.0f,1.0f,1.0f,1.0f);

	// Set shading model
	glShadeModel(GL_SMOOTH);

	// Enable depth test
	glEnable(GL_DEPTH_TEST);

	// Load textures
	if (!load_textures())
	{
		printf("Failed to load textures!\n");
		exit(0);
	}

	// Load shader programs
	dirtProg = load_shaders(vertexFile, fragmentFile);

	// TODO: Associate shader sampler variables with corresponding texture unit
	texSampler[BALL_UNIT] = glGetUniformLocation(dirtProg, "texMap");
	texSampler[DIRT_UNIT] = glGetUniformLocation(dirtProg, "dirtMap");

	// Begin event loop
	glutMainLoop();

	return 0;
}

// Display callback
void display()
{
	// Reset background
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Set modelview matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glRotatef(theta[0],1.0f,0.0f,0.0f);
	glRotatef(theta[1],0.0f,1.0f,0.0f);

	// Render scene without shadows
	render_Scene();

	// Flush buffer
	glFlush();

	// Swap buffers
	glutSwapBuffers();
}

// Scene render function
void render_Scene()
{
	// TODO: Activate shader program
	glUseProgram(dirtProg);

	// TODO: Associate BALL with texture unit 0 (BALL_UNIT)
	glUniform1i(texSampler[BALL_UNIT], BALL_UNIT);
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, tex_ids[BALL]);

	// TODO: Associate DIRT with texture unit 1 (DIRT_UNIT)
	glUniform1i(texSampler[DIRT_UNIT], DIRT_UNIT);
	glActiveTexture(GL_TEXTURE1);
	if (!add_dirt)
	{
		// No dirt - bind BLANK texture
		glBindTexture(GL_TEXTURE_2D, tex_ids[BLANK]);
	}
	else
	{
		// Dirt - bind DIRT texture
		glBindTexture(GL_TEXTURE_2D, tex_ids[DIRT]);
	}

	// TODO: Draw multitextured surface
	glPushMatrix();
		glBegin(GL_POLYGON);
			glColor3f(1.0f,1.0f,1.0f);
			glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 0.0f);
			glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 0.0f);
			glVertex3f(-1.0f, 0.0f, 1.0f);

			glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 0.0f);
			glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 0.0f);
			glVertex3f(1.0f, 0.0f, 1.0f);

			glMultiTexCoord2f(GL_TEXTURE0, 1.0f, 1.0f);
			glMultiTexCoord2f(GL_TEXTURE1, 1.0f, 1.0f);
			glVertex3f(1.0f, 0.0f, -1.0f);

			glMultiTexCoord2f(GL_TEXTURE0, 0.0f, 1.0f);
			glMultiTexCoord2f(GL_TEXTURE1, 0.0f, 1.0f);
			glVertex3f(-1.0f, 0.0f, -1.0f);
		glEnd();
	glPopMatrix();
}

// Keyboard callback
void keyfunc(unsigned char key, int x, int y)
{
	// TODO: <space> toggles dirt
	if (key == ' ')
	{
		add_dirt = !add_dirt;
	}

	// <esc> quits
	if (key == 27)
	{
		exit(0);
	}

	// Redraw screen
	glutPostRedisplay();
}

// Reshape callback
void reshape(int w, int h)
{
	GLfloat ratio;

	// Set new screen extents
	glViewport(0,0,w,h);

	// Set 3D projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// Taller than wide so scale height
	if (w <= h)
	{
		ratio = (GLfloat) h / (GLfloat) w;
		glOrtho(-2.0f,2.0f,-2.0f*ratio,2.0f*ratio,-2.0f,2.0f);
	}
	// Wider than tall so scale width
	else
	{
		ratio = (GLfloat) w / (GLfloat) h;
		glOrtho(-2.0f*ratio,2.0*ratio,-2.0f,2.0f,-2.0f,2.0f);
	}
}

// Mouse callback
void mousefunc(int button, int state, int x, int y)
{
	// Rotate about x and y axes for left button
	if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
	{
		startx = x;
		starty = y;
	}
}

// Mouse move callback
void mousemove(int x, int y)
{
	// Adjust x rotation angle
	theta[0] += (y - starty);
	if (theta[0] > 360.0)
	{
		theta[0] -= 360.0f;
	}
	else if (theta[0] < 0.0f)
	{
		theta[0] += 360.0f;
	}

	// Adjust y rotation angle
	theta[1] += (x - startx);
	if (theta[1] > 360.0)
	{
		theta[1] -= 360.0f;
	}
	else if (theta[1] < 0.0f)
	{
		theta[1] += 360.0f;
	}

	// Update mouse position
	startx = x;
	starty = y;

	// Redraw display
	glutPostRedisplay();
}

// Routine to load textures using SOIL
bool load_textures()
{
	// Load object textures normally
	for (int i=0; i < NO_TEXTURES; i++)
	{
		// Load bitmaps
		tex_ids[i] = SOIL_load_OGL_texture(texture_files[i],SOIL_LOAD_AUTO,SOIL_CREATE_NEW_ID,SOIL_FLAG_MIPMAPS | SOIL_FLAG_INVERT_Y);


		// Set texture properties if successfully loaded
		if (tex_ids[i] != 0)
		{
			// Set scaling filters
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);

			// Set wrapping modes
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
		}
		// Otherwise texture failed to load
		else
		{
			return false;
		}
	}
	return true;
}
